@@ -69,7 +69,7 @@ namespace Nereid
         private ObjectListSourceContents object_view;
         private Label status_label;
         
-        public PlayerInterface () : base (Catalog.GetString ("Banshee Media Player"))
+        public PlayerInterface () : base (Catalog.GetString ("Banshee Media Player"), "player_window", 1024, 700)
         {
         }
         
@@ -85,21 +85,6 @@ namespace Nereid
             Show ();
         }
         
-        protected override void UpdateTitle ()
-        {
-            TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
-            if (track != null) {
-                // Translators: this is the window title when a track is playing
-                //              {0} is the track title, {1} is the artist name
-                Title = String.Format (Catalog.GetString ("{0} by {1}"), 
-                    track.DisplayTrackTitle, track.DisplayArtistName);
-            } else {
-                Title = Catalog.GetString ("Banshee Media Player");
-            }
-            
-            OnTitleChanged ();
-        }
-        
 #region System Overrides 
         
         public override void Dispose ()
@@ -232,15 +217,13 @@ namespace Nereid
         
 #region Events and Logic Setup
         
-        private void ConnectEvents ()
+        protected override void ConnectEvents ()
         {
+            base.ConnectEvents ();
+
             // Service events
             ServiceManager.SourceManager.ActiveSourceChanged += OnActiveSourceChanged;
             ServiceManager.SourceManager.SourceUpdated += OnSourceUpdated;
-            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
-                PlayerEvent.StartOfStream |
-                PlayerEvent.TrackInfoUpdated |
-                PlayerEvent.EndOfStream);
             
             ActionService.TrackActions ["SearchForSameArtistAction"].Activated += OnProgrammaticSearch;
             ActionService.TrackActions ["SearchForSameAlbumAction"].Activated += OnProgrammaticSearch;
@@ -284,54 +267,58 @@ namespace Nereid
         private TrackListModel previous_track_model = null;
         private void OnActiveSourceChanged (SourceEventArgs args)
         {
-            Source source = ServiceManager.SourceManager.ActiveSource;
-
-            view_container.SearchSensitive = source != null && source.CanSearch;
-            
-            if (source == null) {
-                return;
-            }
-            
-            view_container.Title = source.Name;
-            view_container.SearchEntry.Ready = false;
-            view_container.SearchEntry.CancelSearch ();
-
-            if (source.FilterQuery != null) {
-                view_container.SearchEntry.Query = source.FilterQuery;
-                view_container.SearchEntry.ActivateFilter ((int)source.FilterType);
-            }
-
-            if (view_container.Content != null) {
-                view_container.Content.ResetSource ();
-            }
-
-            if (previous_track_model != null) {
-                previous_track_model.Reloaded -= HandleTrackModelReloaded;
-                previous_track_model = null;
-            }
-
-            if (source is ITrackModelSource) {
-                previous_track_model = (source as ITrackModelSource).TrackModel;
-                previous_track_model.Reloaded += HandleTrackModelReloaded;
-            }
-            
-            if (previous_source != null) {
-                previous_source.Properties.PropertyChanged -= OnSourcePropertyChanged;
-            }
-            
-            previous_source = source;
-            previous_source.Properties.PropertyChanged += OnSourcePropertyChanged;
-            
-            UpdateSourceContents (source);
-            
-            UpdateSourceInformation ();
-            view_container.SearchEntry.Ready = true;
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                Source source = ServiceManager.SourceManager.ActiveSource;
+    
+                view_container.SearchSensitive = source != null && source.CanSearch;
+                
+                if (source == null) {
+                    return;
+                }
+                
+                view_container.Title = source.Name;
+                view_container.SearchEntry.Ready = false;
+                view_container.SearchEntry.CancelSearch ();
+    
+                if (source.FilterQuery != null) {
+                    view_container.SearchEntry.Query = source.FilterQuery;
+                    view_container.SearchEntry.ActivateFilter ((int)source.FilterType);
+                }
+    
+                if (view_container.Content != null) {
+                    view_container.Content.ResetSource ();
+                }
+    
+                if (previous_track_model != null) {
+                    previous_track_model.Reloaded -= HandleTrackModelReloaded;
+                    previous_track_model = null;
+                }
+    
+                if (source is ITrackModelSource) {
+                    previous_track_model = (source as ITrackModelSource).TrackModel;
+                    previous_track_model.Reloaded += HandleTrackModelReloaded;
+                }
+                
+                if (previous_source != null) {
+                    previous_source.Properties.PropertyChanged -= OnSourcePropertyChanged;
+                }
+                
+                previous_source = source;
+                previous_source.Properties.PropertyChanged += OnSourcePropertyChanged;
+                
+                UpdateSourceContents (source);
+                
+                UpdateSourceInformation ();
+                view_container.SearchEntry.Ready = true;
+            });
         }
         
         private void OnSourcePropertyChanged (object o, PropertyChangeEventArgs args)
         {
             if (args.PropertyName == "Nereid.SourceContents") {
-                UpdateSourceContents (previous_source);
+                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                    UpdateSourceContents (previous_source);
+                });
             }
         }
         
@@ -392,16 +379,13 @@ namespace Nereid
         private void OnSourceUpdated (SourceEventArgs args)
         {
             if (args.Source == ServiceManager.SourceManager.ActiveSource) {
-                UpdateSourceInformation ();
-                view_container.Title = args.Source.Name;
+                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                    UpdateSourceInformation ();
+                    view_container.Title = args.Source.Name;
+                });
             }
         }
-        
-        private void OnPlayerEvent (PlayerEventArgs args) 
-        {
-            UpdateTitle ();
-        }
-        
+
 #endregion
 
 #region UI Event Handlers
@@ -416,40 +400,10 @@ namespace Nereid
             source.FilterQuery = view_container.SearchEntry.Query;
         }
         
-        private void OnToolbarExposeEvent (object o, ExposeEventArgs args)
-        {
-            Toolbar toolbar = (Toolbar)o;
-
-            // This forces the toolbar to look like it's just a regular part
-            // of the window since the stock toolbar look makes Banshee look ugly.
-            Style.ApplyDefaultBackground (toolbar.GdkWindow, true, State, 
-                args.Event.Area, toolbar.Allocation.X, toolbar.Allocation.Y, 
-                toolbar.Allocation.Width, toolbar.Allocation.Height);
-
-            // Manually expose all the toolbar's children
-            foreach (Widget child in toolbar.Children) {
-                toolbar.PropagateExpose (child, args.Event);
-            }
-        }
-        
 #endregion
 
 #region Implement Interfaces
 
-        // IHasTrackSelection
-        /*public IEnumerable<TrackInfo> GetSelectedTracks ()
-        {
-            return new ModelSelection<TrackInfo> (composite_view.TrackModel, composite_view.TrackView.Selection);
-        }
-
-        public Hyena.Collections.SelectionProxy TrackSelectionProxy {
-            get { return composite_view.TrackView.SelectionProxy; }
-        }
-
-        public DatabaseTrackListModel TrackModel {
-            get { return composite_view.TrackModel as DatabaseTrackListModel; }
-        }*/
-
         // IHasSourceView
         public Source HighlightedSource {
             get { return source_view.HighlightedSource; }
@@ -532,7 +486,7 @@ namespace Nereid
 
         private void HandleTrackModelReloaded (object sender, EventArgs args)
         {
-            UpdateSourceInformation ();
+            Banshee.Base.ThreadAssist.ProxyToMain (UpdateSourceInformation);
         }
 
         private void UpdateSourceInformation ()
