@@ -57,7 +57,7 @@ namespace Banshee.Collection.Database
 
         private long filtered_count;
         private TimeSpan filtered_duration;
-        private long filtered_filesize;
+        private long filtered_filesize, filesize;
         
         private ISortableColumn sort_column;
         private string sort_query;
@@ -77,6 +77,10 @@ namespace Banshee.Collection.Database
             this.provider = provider;
             this.source = source;
         }
+        
+        protected HyenaSqliteConnection Connection {
+            get { return connection; }
+        }
 
         private bool initialized = false;
         public void Initialize (IDatabaseTrackModelCache cache)
@@ -124,8 +128,10 @@ namespace Banshee.Collection.Database
 
         protected virtual void GenerateSortQueryPart ()
         {
-            SortQuery = (SortColumn == null)
-                ? BansheeQuery.GetSort ("Artist", true)
+            SortQuery = (SortColumn == null || SortColumn.SortType == SortType.None)
+                ? (SortColumn != null && source is Banshee.Playlist.PlaylistSource)
+                    ? "CorePlaylistEntries.ViewOrder ASC, CorePlaylistEntries.EntryID ASC" 
+                    : BansheeQuery.GetSort ("Artist", true)
                 : BansheeQuery.GetSort (SortColumn.SortKey, SortColumn.SortType == SortType.Ascending);
         }
 
@@ -137,11 +143,18 @@ namespace Banshee.Collection.Database
                 }
                 
                 if (sort_column == column && sort_column != null) {
-                    sort_column.SortType = sort_column.SortType == SortType.Ascending 
-                        ? SortType.Descending 
-                        : SortType.Ascending;
+                    switch (sort_column.SortType) {
+                        case SortType.Ascending:    sort_column.SortType = SortType.Descending; break;
+                        case SortType.Descending:   sort_column.SortType = SortType.None; break;
+                        case SortType.None:         sort_column.SortType = SortType.Ascending; break;
+                    }
                 }
-            
+                
+                // If we're switching from a different column, or we aren't in a playlist, make sure sort type isn't None
+                if (sort_column != null && (sort_column != column || !(source is Banshee.Playlist.PlaylistSource)) && sort_column.SortType == SortType.None) {
+                    sort_column.SortType = SortType.Ascending;
+                }
+                
                 sort_column = column;
             
                 GenerateSortQueryPart ();
@@ -159,6 +172,7 @@ namespace Banshee.Collection.Database
         {
             cache.Clear ();
             count = 0;
+            filesize = 0;
             filtered_count = 0;
             OnCleared ();
         }
@@ -168,13 +182,13 @@ namespace Banshee.Collection.Database
             if (cache == null) {
                 Log.ErrorFormat ("Called invalidate cache for {0}'s track model, but cache is null", source);
             } else {
-                cache.ClearManagedCache ();
+                cache.Clear ();
                 OnReloaded ();
             }
         }
 
         private string unfiltered_query;
-        protected string UnfilteredQuery {
+        public string UnfilteredQuery {
             get {
                 return unfiltered_query ?? unfiltered_query = String.Format (
                     "FROM {0}{1} WHERE {2} {3}",
@@ -185,12 +199,16 @@ namespace Banshee.Collection.Database
             }
         }
 
-        private void UpdateUnfilteredAggregates ()
+        public virtual void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
-                "SELECT COUNT(*) {0}", UnfilteredQuery
+                "SELECT COUNT(*), SUM(CoreTracks.FileSize) {0}", UnfilteredQuery
             ));
-            count = connection.Query<long> (count_command);
+            
+            using (HyenaDataReader reader = new HyenaDataReader (connection.Query (count_command))) {
+                count = reader.Get<long> (0);
+                filesize = reader.Get<long> (1);
+            }
         }
 
         public override void Reload ()
@@ -214,10 +232,10 @@ namespace Banshee.Collection.Database
 
                 List<IFilterListModel> reload_models = new List<IFilterListModel> ();
                 bool found = (reloadTrigger == null);
-                foreach (IFilterListModel model in source.FilterModels) {
+                foreach (IFilterListModel filter in source.CurrentFilters) {
                     if (found) {
-                        reload_models.Add (model);
-                    } else if (model == reloadTrigger) {
+                        reload_models.Add (filter);
+                    } else if (filter == reloadTrigger) {
                         found = true;
                     }
                 }
@@ -232,8 +250,8 @@ namespace Banshee.Collection.Database
                     }
                     
                     bool have_filters = false;
-                    foreach (IFilterListModel model in source.FilterModels) {
-                        have_filters |= !model.Selection.AllSelected;
+                    foreach (IFilterListModel filter in source.CurrentFilters) {
+                        have_filters |= !filter.Selection.AllSelected;
                     }
                     
                     // Unless both artist/album selections are "all" (eg unfiltered), reload
@@ -268,11 +286,11 @@ namespace Banshee.Collection.Database
             qb.Append (UnfilteredQuery);
             
             if (with_filters) {
-                foreach (IFilterListModel model in source.FilterModels) {
-                    string filter = model.GetSqlFilter ();
-                    if (filter != null) {
+                foreach (IFilterListModel filter in source.CurrentFilters) {
+                    string filter_sql = filter.GetSqlFilter ();
+                    if (filter_sql != null) {
                         qb.Append (" AND ");
-                        qb.Append (filter);
+                        qb.Append (filter_sql);
                     }
                 }
             }
@@ -293,12 +311,16 @@ namespace Banshee.Collection.Database
 
         public override int IndexOf (TrackInfo track)
         {
-            DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
-            if (db_track == null || db_track.CacheModelId != CacheId) {
-                return -1;
+            lock (this) {
+                return (int) cache.IndexOf (track as DatabaseTrackInfo);
             }
+        }
 
-            return (int) cache.IndexOf ((int)db_track.CacheEntryId);
+        public int IndexOfFirst (TrackInfo track)
+        {
+            lock (this) {
+                return IndexOf (cache.GetSingle ("AND MetadataHash = ? ORDER BY OrderID", track.MetadataHash));
+            }
         }
 
         private DateTime random_began_at = DateTime.MinValue;
@@ -345,8 +367,13 @@ namespace Banshee.Collection.Database
             get { return filtered_filesize; }
         }
         
+        public long UnfilteredFileSize {
+            get { return filesize; }
+        }
+        
         public int UnfilteredCount {
             get { return (int) count; }
+            set { count = value; }
         }
 
         public string UserQuery {
@@ -364,7 +391,9 @@ namespace Banshee.Collection.Database
             set { 
                 forced_sort_query = value != null;
                 sort_query = value;
-                cache.Clear ();
+                if (cache != null) {
+                    cache.Clear ();
+                }
             }
         }
 
@@ -392,9 +421,11 @@ namespace Banshee.Collection.Database
         
         public void AddCondition (string part)
         {
-            condition = condition == null ? part : String.Format ("{0} AND {1}", condition, part);
+            if (!String.IsNullOrEmpty (part)) {
+                condition = condition == null ? part : String.Format ("{0} AND {1}", condition, part);
+            }
         }
-
+        
         public string Condition {
             get { return condition; }
         }
@@ -470,5 +501,7 @@ namespace Banshee.Collection.Database
         public string ReloadFragment {
             get { return reload_fragment; }
         }
+        
+        public bool CachesValues { get { return false; } }
     }
 }
