@@ -78,12 +78,17 @@ namespace Banshee.Playlist
             get { return true; }
         }
 
+        private bool is_temporary = false;
+        public bool IsTemporary {
+            get { return is_temporary; }
+            set { is_temporary = value; }
+        }
+
         public int? DbId {
             get { return dbid; }
             protected set {
                 if (value != null && value != dbid) {
                     dbid = value;
-                    AfterInitialized ();
                 }
             }
         }
@@ -122,15 +127,20 @@ namespace Banshee.Playlist
             }
         }
 
-        public AbstractPlaylistSource (string generic_name, string name, int primarySourceId)
-            : this (generic_name, name, null, -1, 0, primarySourceId)
+        public AbstractPlaylistSource (string generic_name, string name, int primarySourceId) : base ()
         {
+            GenericName = generic_name;
+            Name = name;
+            primary_source_id = primarySourceId;
         }
 
-        public AbstractPlaylistSource (string generic_name, string name, int? dbid, int sortColumn, int sortType, int primarySourceId)
-            : base (generic_name, name, Convert.ToString (dbid), 500)
+        public AbstractPlaylistSource (string generic_name, string name, int dbid, int sortColumn, int sortType, int primarySourceId, bool is_temp)
+            : base (generic_name, name, dbid.ToString (), 500)
         {
+            DbId = dbid;
+            IsTemporary = is_temp;
             this.primary_source_id = primarySourceId;
+            AfterInitialized ();
         }
 
         protected override void AfterInitialized ()
@@ -140,7 +150,6 @@ namespace Banshee.Playlist
             DatabaseTrackModel.JoinColumn = "TrackID";
             DatabaseTrackModel.CachesJoinTableEntries = CachesJoinTableEntries;
             DatabaseTrackModel.AddCondition (String.Format (TrackCondition, dbid));
-
             base.AfterInitialized ();
         }
 
@@ -151,19 +160,39 @@ namespace Banshee.Playlist
         }
 
         public override bool CanRename {
-            get { return true; }
+            get { return (Parent is PrimarySource) ? !(Parent as PrimarySource).PlaylistsReadOnly : true; }
         }
 
         public override bool CanSearch {
             get { return true; }
         }
 
+        public virtual bool CanUnmap {
+            get { return (Parent is PrimarySource) ? !(Parent as PrimarySource).PlaylistsReadOnly : true; }
+        }
+
+        public bool ConfirmBeforeUnmap {
+            get { return true; }
+        }
+
+        // We can delete tracks only if our parent can
+        public override bool CanDeleteTracks {
+            get {
+                DatabaseSource ds = Parent as DatabaseSource;
+                return ds != null && ds.CanDeleteTracks;
+            }
+        }
+
         public override void Save ()
         {
-            if (dbid == null || dbid <= 0)
+            if (dbid == null || dbid <= 0) {
                 Create ();
-            else
+                TypeUniqueId = dbid.ToString ();
+                Initialize ();
+                AfterInitialized ();
+            } else {
                 Update ();
+            }
         }
 
         // Have our parent handle deleting tracks
