@@ -12,9 +12,9 @@ using Banshee.Configuration;
 using Banshee.ServiceStack;
 using Banshee.Database;
 using Banshee.Sources;
-using Banshee.Library;
 using Banshee.Playlists.Formats;
 using Banshee.Collection;
+using Banshee.Collection.Database;
 
 namespace Banshee.Playlist
 {
@@ -106,7 +106,38 @@ namespace Banshee.Playlist
         }
         
         public static string [] ImportPlaylist (string playlistUri)
-        {            
+        {
+            return ImportPlaylist (playlistUri, null);
+        }
+
+        public static string [] ImportPlaylist (string playlistUri, Uri baseUri)
+        {
+            IPlaylistFormat playlist = Load (playlistUri, baseUri);
+            List<string> uris = new List<string> ();
+            if (playlist != null) {
+                foreach (Dictionary<string, object> element in playlist.Elements) {
+                    uris.Add (((Uri)element["uri"]).AbsoluteUri);
+                }
+            }
+            return uris.ToArray ();
+        }
+        
+        public static bool PathHasPlaylistExtension (string playlistUri)
+        {
+            if (System.IO.Path.HasExtension (playlistUri)) {
+                string extension = System.IO.Path.GetExtension (playlistUri).ToLower ();
+                foreach (PlaylistFormatDescription format in PlaylistFileUtil.ExportFormats) {
+                    if (extension.Equals ("." + format.FileExtension)) {
+                        return true;
+                    }
+                }
+            }
+            
+            return false;
+        }
+
+        public static IPlaylistFormat Load (string playlistUri, Uri baseUri)
+        {
             PlaylistFormatDescription [] formats = PlaylistFileUtil.ExportFormats;            
             
             // If the file has an extenstion, rearrange the format array so that the 
@@ -131,25 +162,25 @@ namespace Banshee.Playlist
                 }
             }
             
-            List<string> uris = new List<string> ();
-                
             foreach (PlaylistFormatDescription format in formats) {
                 try {
                     IPlaylistFormat playlist = (IPlaylistFormat)Activator.CreateInstance (format.Type);
+                    playlist.BaseUri = baseUri;
                     playlist.Load (Banshee.IO.File.OpenRead (new SafeUri (playlistUri)), true);
-                    foreach (Dictionary<string, object> element in playlist.Elements) {
-                        uris.Add (((Uri)element["uri"]).AbsoluteUri);
-                    }
-                    break;
-                } catch (InvalidPlaylistException) {                    
-                    continue;
-                }      
+                    return playlist;
+                } catch (InvalidPlaylistException) {
+                }
             }
         
-            return uris.ToArray ();
+            return null;
         }
         
         public static void ImportPlaylistToLibrary (string path)
+        {
+            ImportPlaylistToLibrary (path, ServiceManager.SourceManager.MusicLibrary, null);
+        }
+        
+        public static void ImportPlaylistToLibrary (string path, PrimarySource source, DatabaseImportManager importer)
         {
             try {
                 SafeUri uri = new SafeUri (path);
@@ -165,7 +196,9 @@ namespace Banshee.Playlist
                         uris.Add (((Uri)element["uri"]).LocalPath);
                     }
                     
-                    ImportPlaylistWorker worker = new ImportPlaylistWorker (System.IO.Path.GetFileNameWithoutExtension (uri.LocalPath), uris.ToArray ());
+                    ImportPlaylistWorker worker = new ImportPlaylistWorker (
+                        System.IO.Path.GetFileNameWithoutExtension (uri.LocalPath), 
+                        uris.ToArray (), source, importer);
                     worker.Import ();
                 }
             } catch (Exception e) {
@@ -178,19 +211,25 @@ namespace Banshee.Playlist
     {
         private string [] uris;
         private string name;
-        private LibraryImportManager importer;
         
-        public ImportPlaylistWorker (string name, string [] uris)
+        private PrimarySource source;
+        private DatabaseImportManager importer;
+        
+        public ImportPlaylistWorker (string name, string [] uris, PrimarySource source, DatabaseImportManager importer)
         {
             this.name = name;
             this.uris = uris;
+            this.source = source;
+            this.importer = importer;
         }
         
         public void Import ()
         {
             try {
-                importer = new LibraryImportManager ();
-                importer.ImportFinished += CreatePlaylist;
+                if (importer == null) {
+                    importer = new Banshee.Library.LibraryImportManager ();
+                }
+                importer.Finished += CreatePlaylist;
                 importer.Enqueue (uris);
             } catch (PlaylistImportCanceledException e) {
                 Hyena.Log.Exception (e);
@@ -200,16 +239,17 @@ namespace Banshee.Playlist
         private void CreatePlaylist (object o, EventArgs args)
         {
             try {
-                PlaylistSource playlist = new PlaylistSource (name, ServiceManager.SourceManager.MusicLibrary.DbId);
+                PlaylistSource playlist = new PlaylistSource (name, source.DbId);
                 playlist.Save ();
-                ServiceManager.SourceManager.MusicLibrary.AddChildSource (playlist);
+                source.AddChildSource (playlist);
 
                 HyenaSqliteCommand insert_command = new HyenaSqliteCommand (String.Format (
                     @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID) VALUES ({0}, ?)", playlist.DbId));
 
                 //ServiceManager.DbConnection.BeginTransaction ();
                 foreach (string uri in uris) {
-                    int track_id = LibrarySource.GetTrackIdForUri (uri);
+                    // FIXME: Does the following call work if the source is just a PrimarySource (not LibrarySource)?
+                    int track_id = Banshee.Library.LibrarySource.GetTrackIdForUri (uri);
                     if (track_id > 0) {
                         ServiceManager.DbConnection.Execute (insert_command, track_id);
                     }
