@@ -36,13 +36,13 @@ using Banshee.Library;
 using Banshee.Playlist;
 using Banshee.SmartPlaylist;
 using Banshee.Sources;
+using Banshee.Base;
 
 namespace Banshee.ServiceStack
 {    
     public delegate bool ShutdownRequestHandler ();
     public delegate bool TimeoutHandler ();
     public delegate bool IdleHandler ();
-    public delegate void InvokeHandler ();
     public delegate bool IdleTimeoutRemoveHandler (uint id);
     public delegate uint TimeoutImplementationHandler (uint milliseconds, TimeoutHandler handler); 
     public delegate uint IdleImplementationHandler (IdleHandler handler);
@@ -71,12 +71,21 @@ namespace Banshee.ServiceStack
         private static Stack<Client> running_clients = new Stack<Client> ();
         private static bool shutting_down;
 
+        public static void Initialize ()
+        {
+            ServiceManager.DefaultInitialize ();
+        }
+
         public static void Run ()
         {
             Banshee.Base.PlatformHacks.TrapMonoJitSegv ();
             
             Catalog.Init (Application.InternalName, System.IO.Path.Combine (
                 Banshee.Base.Paths.InstalledApplicationDataRoot, "locale"));
+                
+            if (!DBusConnection.ConnectTried) {
+                DBusConnection.Connect ();
+            }
 
             ServiceManager.Run ();
 
@@ -159,12 +168,7 @@ namespace Banshee.ServiceStack
         {
             RunIdle (delegate { handler (); return false; });
         }
-        
-        public static void Invoke (EventHandler handler)
-        {
-            RunIdle (delegate { handler (null, EventArgs.Empty); return false; });
-        }
-        
+
         public static uint RunIdle (IdleHandler handler)
         {
             if (idle_handler == null) {
