@@ -106,18 +106,33 @@ namespace Banshee.SmartPlaylist
         }
 
         // Custom properties
+        private List<QueryField> relevant_fields = new List<QueryField> ();
         private QueryNode condition;
         public QueryNode ConditionTree {
             get { return condition; }
             set {
                 condition = value;
+                relevant_fields.Clear ();
                 if (condition != null) {
                     condition_sql = condition.ToSql (BansheeQuery.FieldSet);
                     condition_xml = condition.ToXml (BansheeQuery.FieldSet);
+                    
+                    foreach (QueryTermNode term in condition.GetTerms ()) {
+                        if (!relevant_fields.Contains (term.Field))
+                            relevant_fields.Add (term.Field);
+                    }
                 }
             }
         }
 
+        private string additional_conditions;
+        public void AddCondition (string part)
+        {
+            if (!String.IsNullOrEmpty (part)) {
+                additional_conditions = additional_conditions == null ? part : String.Format ("{0} AND {1}", additional_conditions, part);
+            }
+        }
+
         private string condition_sql;
         public virtual string ConditionSql {
             get { return condition_sql; }
@@ -175,43 +190,39 @@ namespace Banshee.SmartPlaylist
 
 #region Constructors
 
-        public SmartPlaylistSource (string name, int primarySourceId) : this (null, name, String.Empty, String.Empty, String.Empty, String.Empty, primarySourceId, 0)
+        public SmartPlaylistSource (string name, int primarySourceId) : base (generic_name, name, primarySourceId)
         {
+            SetProperties ();
         }
 
         public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, int primarySourceId)
-            : base (generic_name, name, null, -1, 0, primarySourceId)
+            : this (name, primarySourceId)
         {
             ConditionTree = condition;
             QueryOrder = order;
             Limit = limit;
             LimitValue = limit_value;
 
-            InstallProperties ();
+            SetProperties ();
             UpdateDependencies ();
         }
 
         // For existing smart playlists that we're loading from the database
-        protected SmartPlaylistSource (int? dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, int primarySourceId, int count) :
-            base (generic_name, name, dbid, -1, 0, primarySourceId)
+        protected SmartPlaylistSource (int dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, int primarySourceId, int count, bool is_temp) :
+            base (generic_name, name, dbid, -1, 0, primarySourceId, is_temp)
         {
             ConditionXml = condition_xml;
             QueryOrder = BansheeQuery.FindOrder (order_by);
-            
             Limit = BansheeQuery.FindLimit (limit_criterion);
-
             LimitValue = new IntegerQueryValue ();
             LimitValue.ParseUserQuery (limit_number);
-
-            DbId = dbid;
             SavedCount = count;
 
-            InstallProperties ();
-
+            SetProperties ();
             UpdateDependencies ();
         }
 
-        protected void InstallProperties ()
+        private void SetProperties ()
         {
             Properties.SetString ("Icon.Name", "source-smart-playlist");
             Properties.SetString ("SourcePropertiesActionLabel", properties_label);
@@ -264,17 +275,28 @@ namespace Banshee.SmartPlaylist
 
 #region AbstractPlaylist overrides
 
+        protected override void AfterInitialized ()
+        {
+            base.AfterInitialized ();
+
+            if (PrimarySource != null) {
+                PrimarySource.TracksAdded += HandleTracksAdded;
+                PrimarySource.TracksChanged += HandleTracksChanged;
+                PrimarySource.TracksDeleted += HandleTracksDeleted;
+            }
+        }
+
         protected override void Create ()
         {
             DbId = ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
                 INSERT INTO CoreSmartPlaylists
-                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID)
-                    VALUES (?, ?, ?, ?, ?, ?)",
+                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, IsTemporary)
+                    VALUES (?, ?, ?, ?, ?, ?, ?)",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                PrimarySourceId
+                PrimarySourceId, IsTemporary
             ));
             UpdateDependencies ();
         }
@@ -288,17 +310,32 @@ namespace Banshee.SmartPlaylist
                         OrderBy = ?,
                         LimitNumber = ?,
                         LimitCriterion = ?,
-                        CachedCount = ?
+                        CachedCount = ?,
+                        IsTemporary = ?
                     WHERE SmartPlaylistID = ?",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                Count, DbId
+                Count, IsTemporary, DbId
             ));
             UpdateDependencies ();
         }
 
+        protected override bool NeedsReloadWhenFieldChanged (Hyena.Query.QueryField field)
+        {
+            if (base.NeedsReloadWhenFieldChanged (field))
+                return true;
+
+            if (QueryOrder != null && QueryOrder.Field == field)
+                return true;
+
+            if (relevant_fields.Contains (field))
+                return true;
+
+            return false;
+        }
+
 #endregion
 
 #region DatabaseSource overrides
@@ -324,11 +361,11 @@ namespace Banshee.SmartPlaylist
             string reload_str = String.Format (
                 @"DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = {0};
                   INSERT INTO CoreSmartPlaylistEntries 
-                    SELECT NULL, {0} as SmartPlaylistID, TrackId
-                        FROM CoreTracks, CoreArtists, CoreAlbums
-                        WHERE CoreTracks.ArtistID = CoreArtists.ArtistID AND CoreTracks.AlbumID = CoreAlbums.AlbumID AND CoreTracks.PrimarySourceID = {1}
-                        {2} {3} {4}",
-                DbId, PrimarySourceId, PrependCondition("AND"), OrderSql, LimitSql
+                    SELECT NULL, {0} as SmartPlaylistID, TrackId FROM {1}
+                        WHERE {2} AND CoreTracks.PrimarySourceID = {3}
+                        {4} {5} {6}",
+                DbId, DatabaseTrackInfo.Provider.From, DatabaseTrackInfo.Provider.Where,
+                PrimarySourceId, PrependCondition ("AND"), OrderSql, LimitSql
             );
             ServiceManager.DbConnection.Execute (reload_str);
 
@@ -384,18 +421,10 @@ namespace Banshee.SmartPlaylist
                 ));
             }
 
-            Remove ();
+            ThreadAssist.ProxyToMain (Remove);
             return true;
         }
 
-        public virtual bool CanUnmap {
-            get { return true; }
-        }
-
-        public bool ConfirmBeforeUnmap {
-            get { return true; }
-        }
-
         public bool CanRefresh {
             get { return QueryOrder == BansheeQuery.RandomOrder; }
         }
@@ -411,8 +440,12 @@ namespace Banshee.SmartPlaylist
         protected override void HandleTracksChanged (Source sender, TrackEventArgs args)
         {
             if (args.When > last_updated) {
-                last_updated = args.When;
-                RefreshAndReload ();
+                if (NeedsReloadWhenFieldsChanged (args.ChangedFields)) {
+                    last_updated = args.When;
+                    RefreshAndReload ();
+                } else {
+                    InvalidateCaches ();
+                }
             }
         }
 
@@ -421,7 +454,6 @@ namespace Banshee.SmartPlaylist
             if (args.When > last_removed) {
                 last_removed = args.When;
                 RefreshAndReload ();
-                Reload ();
                 /*if (ServiceManager.DbConnection.Query<int> (count_removed_command, last_removed) > 0) {
                     if (Limit == null) {
                         //track_model.UpdateAggregates ();
@@ -437,35 +469,33 @@ namespace Banshee.SmartPlaylist
         public override void SetParentSource (Source parent)
         {
             base.SetParentSource (parent);
-
-            PrimarySource primary = parent as PrimarySource;
-            if (primary != null) {
-                primary.TracksAdded += HandleTracksAdded;
-                primary.TracksChanged += HandleTracksChanged;
-                primary.TracksDeleted += HandleTracksDeleted;
-            }
         }
 
 #endregion
 
         private string PrependCondition (string with)
         {
-            return String.IsNullOrEmpty (ConditionSql) ? " " : String.Format ("{0} ({1})", with, ConditionSql);
+            string sql = String.IsNullOrEmpty (ConditionSql) ? " " : String.Format ("{0} ({1})", with, ConditionSql);
+            if (!String.IsNullOrEmpty (additional_conditions)) {
+                sql = String.IsNullOrEmpty (sql) ? additional_conditions : String.Format ("{0} AND ({1})", sql, additional_conditions);
+            }
+            return sql;
         }
 
         public static IEnumerable<SmartPlaylistSource> LoadAll (int primary_id)
         {
-            using (IDataReader reader = ServiceManager.DbConnection.Query (
-                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount
-                    FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", primary_id)) {
+            ClearTemporary ();
+            using (HyenaDataReader reader = new HyenaDataReader (ServiceManager.DbConnection.Query (
+                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount, IsTemporary
+                    FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", primary_id))) {
                 while (reader.Read ()) {
                     SmartPlaylistSource playlist = null;
                     try {
                         playlist = new SmartPlaylistSource (
-                            Convert.ToInt32 (reader[0]), reader[1] as string,
-                            reader[2] as string, reader[3] as string,
-                            reader[4] as string, reader[5] as string,
-                            Convert.ToInt32 (reader[6]), Convert.ToInt32 (reader[7])
+                            reader.Get<int> (0), reader.Get<string> (1),
+                            reader.Get<string> (2), reader.Get<string> (3),
+                            reader.Get<string> (4), reader.Get<string> (5),
+                            reader.Get<int> (6), reader.Get<int> (7), reader.Get<bool> (8)
                         );
                     } catch (Exception e) {
                         Log.Warning ("Ignoring Smart Playlist", String.Format ("Caught error: {0}", e), false);
@@ -478,6 +508,20 @@ namespace Banshee.SmartPlaylist
             }
         }
 
+        private static bool temps_cleared = false;
+        private static void ClearTemporary ()
+        {
+            if (!temps_cleared) {
+                temps_cleared = true;
+                ServiceManager.DbConnection.Execute (@"
+                    BEGIN TRANSACTION;
+                        DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID IN (SELECT SmartPlaylistID FROM CoreSmartPlaylists WHERE IsTemporary = 1);
+                        DELETE FROM CoreSmartPlaylists WHERE IsTemporary = 1;
+                    COMMIT TRANSACTION"
+                );
+            }
+        }
+
         private static void HandleSourceAdded (SourceEventArgs args)
         {
             SmartPlaylistSource playlist = args.Source as SmartPlaylistSource;
