@@ -28,6 +28,7 @@
 //
 
 using System;
+using System.Text;
 using System.Collections.Generic;
 
 using Mono.Unix;
@@ -43,28 +44,34 @@ using Banshee.ServiceStack;
 using Banshee.Sources;
 using Banshee.Collection;
 using Banshee.Collection.Database;
+using Banshee.Configuration;
 using Banshee.Query;
 
 namespace Banshee.Sources
 {
-    public abstract class DatabaseSource : Source, ITrackModelSource, IDurationAggregator, IFileSizeAggregator
+    public abstract class DatabaseSource : Source, ITrackModelSource, IFilterableSource, IDurationAggregator, IFileSizeAggregator
     {
+        public event EventHandler FiltersChanged;
+
         protected delegate void TrackRangeHandler (DatabaseTrackListModel model, RangeCollection.Range range);
 
         protected DatabaseTrackListModel track_model;
         protected DatabaseAlbumListModel album_model;
         protected DatabaseArtistListModel artist_model;
-
-        protected RateLimiter reload_limiter;
         
-        protected string type_unique_id;
-        protected override string TypeUniqueId {
-            get { return type_unique_id; }
+        private DatabaseQueryFilterModel<string> genre_model;
+        protected DatabaseQueryFilterModel<string> GenreModel {
+            get {
+                return genre_model ?? 
+                    genre_model = new Banshee.Collection.Database.DatabaseQueryFilterModel<string> (this, DatabaseTrackModel, ServiceManager.DbConnection,
+                        Catalog.GetString ("All Genres ({0})"), UniqueId, BansheeQuery.GenreField, "Genre");
+            }
         }
+
+        private RateLimiter reload_limiter;
         
-        public DatabaseSource (string generic_name, string name, string id, int order) : base (generic_name, name, order)
+        public DatabaseSource (string generic_name, string name, string id, int order) : base (generic_name, name, order, id)
         {
-            type_unique_id = id;
             DatabaseSourceInitialize ();
         }
 
@@ -72,6 +79,13 @@ namespace Banshee.Sources
         {
         }
 
+        public void UpdateCounts ()
+        {
+            DatabaseTrackModel.UpdateUnfilteredAggregates ();
+            ever_counted = true;
+            OnUpdated ();
+        }
+
         public abstract void Save ();
 
         protected override void Initialize ()
@@ -84,11 +98,11 @@ namespace Banshee.Sources
             get { return true; }
         }
 
-        protected DatabaseTrackListModel DatabaseTrackModel {
+        public DatabaseTrackListModel DatabaseTrackModel {
             get {
                 return track_model ?? track_model = new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, this);
             }
-            set { track_model = value; }
+            protected set { track_model = value; }
         }
 
         private IDatabaseTrackModelCache track_cache;
@@ -107,15 +121,31 @@ namespace Banshee.Sources
         private void DatabaseSourceInitialize ()
         {
             InitializeTrackModel ();
+            
+            current_filters_schema = CreateSchema<string[]> ("current_filters");
 
             if (HasArtistAlbum) {
                 artist_model = new DatabaseArtistListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
                 album_model = new DatabaseAlbumListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
+                
+                AvailableFilters.Add (GenreModel);
+                AvailableFilters.Add (artist_model);
+                AvailableFilters.Add (album_model);
+                
+                DefaultFilters.Add (GenreModel);
+                DefaultFilters.Add (artist_model);
+                DefaultFilters.Add (album_model);
             }
 
             reload_limiter = new RateLimiter (RateLimitedReload);
         }
 
+        protected virtual void AfterInitialized ()
+        {
+            DatabaseTrackModel.Initialize (TrackCache);
+            OnSetupComplete ();
+        }
+
         protected virtual void InitializeTrackModel ()
         {
         }
@@ -147,10 +177,9 @@ namespace Banshee.Sources
 
             ISortableColumn sort_column = (TrackModel is DatabaseTrackListModel)
                 ? (TrackModel as DatabaseTrackListModel).SortColumn : null;
+
             // If it's the field we're sorting by, then yes, we care
-            // FIXME this Contains is very hacky, we should link the ISortableColumn to a field and/or a
-            // QueryOrder object
-            if (sort_column != null && field.Column.Contains (sort_column.SortKey)) {
+            if (sort_column != null && sort_column.Field == field) {
                 return true;
             }
 
@@ -173,7 +202,7 @@ namespace Banshee.Sources
 #region Public Properties
 
         public override int Count {
-            get { return ever_reloaded ? DatabaseTrackModel.UnfilteredCount : SavedCount; }
+            get { return ever_counted ? DatabaseTrackModel.UnfilteredCount : SavedCount; }
         }
 
         public override int FilteredCount {
@@ -222,20 +251,14 @@ namespace Banshee.Sources
             get { return DatabaseTrackModel; }
         }
         
-        public virtual IEnumerable<IFilterListModel> FilterModels {
-            get {
-                if (artist_model != null)
-                    yield return artist_model;
-                    
-                if (album_model != null)
-                    yield return album_model;
-            }
-        }
-        
         public virtual bool ShowBrowser { 
             get { return true; }
         }
 
+        public virtual bool Indexable {
+            get { return false; }
+        }
+
         private int saved_count;
         protected int SavedCount {
             get { return saved_count; }
@@ -248,12 +271,100 @@ namespace Banshee.Sources
         }
 
 #endregion
+        
+#region Filters (aka Browsers)
+        
+        private IList<IFilterListModel> available_filters;
+        public IList<IFilterListModel> AvailableFilters {
+            get { return available_filters ?? available_filters = new List<IFilterListModel> (); }
+            protected set { available_filters = value; }
+        }
+        
+        private IList<IFilterListModel> default_filters;
+        public IList<IFilterListModel> DefaultFilters {
+            get { return default_filters ?? default_filters = new List<IFilterListModel> (); }
+            protected set { default_filters = value; }
+        }
+        
+        private IList<IFilterListModel> current_filters;
+        public IList<IFilterListModel> CurrentFilters {
+            get {
+                if (current_filters == null) {
+                    current_filters = new List<IFilterListModel> ();
+                    string [] current = current_filters_schema.Get ();
+                    if (current != null) {
+                        foreach (string filter_name in current) {
+                            foreach (IFilterListModel filter in AvailableFilters) {
+                                if (filter.FilterName == filter_name) {
+                                    current_filters.Add (filter);
+                                    break;
+                                }
+                            }
+                        }
+                    } else {
+                        foreach (IFilterListModel filter in DefaultFilters) {
+                            current_filters.Add (filter);
+                        }
+                    }
+                }
+                return current_filters;
+            }
+            protected set { current_filters = value; }
+        }
+        
+        public void ReplaceFilter (IFilterListModel old_filter, IFilterListModel new_filter)
+        {
+            int i = current_filters.IndexOf (old_filter);
+            if (i != -1) {
+                current_filters[i] = new_filter;
+                SaveCurrentFilters ();
+            }
+        }
+        
+        public void AppendFilter (IFilterListModel filter)
+        {
+            if (current_filters.IndexOf (filter) == -1) {
+                current_filters.Add (filter);
+                SaveCurrentFilters ();
+            }
+        }
+        
+        public void RemoveFilter (IFilterListModel filter)
+        {
+            if (current_filters.Remove (filter)) {
+                SaveCurrentFilters ();
+            }
+        }
+        
+        private void SaveCurrentFilters ()
+        {
+            Reload ();
+            if (current_filters == null) {
+                current_filters_schema.Set (null);
+            } else {
+                string [] filters = new string [current_filters.Count];
+                int i = 0;
+                foreach (IFilterListModel filter in CurrentFilters) {
+                    filters[i++] = filter.FilterName;
+                }
+                current_filters_schema.Set (filters);
+            }
+            
+            EventHandler handler = FiltersChanged;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
+        }
+        
+        private SchemaEntry<string[]> current_filters_schema;
+
+#endregion
 
 #region Public Methods
 
         public virtual void Reload ()
         {
-            ever_reloaded = true;
+            ever_counted = ever_reloaded = true;
             reload_limiter.Execute ();
         }
 
@@ -314,6 +425,9 @@ namespace Banshee.Sources
                 return false;
 
             DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
+            if (model == null) {
+                return false;
+            }
             WithTrackSelection (model, AddTrackRange);
             OnTracksAdded ();
             OnUserNotifyUpdated ();
@@ -366,13 +480,13 @@ namespace Banshee.Sources
 
         public override void MergeSourceInput (Source source, SourceMergeType mergeType)
         {
-            if (mergeType == SourceMergeType.Source) {
+            if (mergeType == SourceMergeType.Source || mergeType == SourceMergeType.All) {
                 AddAllTracks (source);
             } else if (mergeType == SourceMergeType.ModelSelection) {
                 AddSelectedTracks (source);
             }
         }
-
+        
 #endregion
         
 #region Protected Methods
@@ -454,18 +568,20 @@ namespace Banshee.Sources
             }
         }
 
-        protected virtual void AfterInitialized ()
-        {
-            DatabaseTrackModel.Initialize (TrackCache);
-            OnSetupComplete ();
-        }
-
-        private bool ever_reloaded = false;
+        private bool ever_reloaded = false, ever_counted = false;
         public override void Activate ()
         {
             if (!ever_reloaded)
                 Reload ();
         }
+        
+        public override void Deactivate ()
+        {
+            DatabaseTrackModel.InvalidateCache ();
+            foreach (IFilterListModel filter in AvailableFilters) {
+                filter.InvalidateCache ();
+            }
+        }
 
         protected virtual void RemoveTrackRange (DatabaseTrackListModel model, RangeCollection.Range range)
         {
@@ -523,13 +639,10 @@ namespace Banshee.Sources
         protected void InvalidateCaches ()
         {
             track_model.InvalidateCache ();
-            
-            // TODO invalidate cache on all FilterModels
-            if (artist_model != null)
-                artist_model.InvalidateCache ();
 
-            if (album_model != null)
-                album_model.InvalidateCache ();
+            foreach (IFilterListModel filter in CurrentFilters) {
+                filter.InvalidateCache ();
+            }
         }
 
         protected virtual void PruneArtistsAlbums ()
