@@ -62,12 +62,17 @@ namespace Banshee.Sources
         public event SourceEventHandler ChildSourceRemoved;
 
         public delegate void OpenPropertiesDelegate ();
-        
-        protected Source (string generic_name, string name, int order)
+
+        protected Source (string generic_name, string name, int order) : this (generic_name, name, order, null)
+        {
+        }
+
+        protected Source (string generic_name, string name, int order, string type_unique_id)
         {
             GenericName = generic_name;
             Name = name;
             Order = order;
+            TypeUniqueId = type_unique_id;
 
             SourceInitialize ();
         }
@@ -387,22 +392,26 @@ namespace Banshee.Sources
     
         protected virtual void OnChildSourceAdded (Source source)
         {
-            SourceEventHandler handler = ChildSourceAdded;
-            if (handler != null) {
-                SourceEventArgs args = new SourceEventArgs ();
-                args.Source = source;
-                handler (args);
-            }
+            ThreadAssist.ProxyToMain (delegate {
+                SourceEventHandler handler = ChildSourceAdded;
+                if (handler != null) {
+                    SourceEventArgs args = new SourceEventArgs ();
+                    args.Source = source;
+                    handler (args);
+                }
+            });
         }
         
         protected virtual void OnChildSourceRemoved (Source source)
         {
-            SourceEventHandler handler = ChildSourceRemoved;
-            if (handler != null) {
-                SourceEventArgs args = new SourceEventArgs ();
-                args.Source = source;
-                handler (args);
-            }
+            ThreadAssist.ProxyToMain (delegate {
+                SourceEventHandler handler = ChildSourceRemoved;
+                if (handler != null) {
+                    SourceEventArgs args = new SourceEventArgs ();
+                    args.Source = source;
+                    handler (args);
+                }
+            });
         }
         
         protected virtual void OnUpdated ()
@@ -455,10 +464,19 @@ namespace Banshee.Sources
         
         private string unique_id;
         public string UniqueId {
-            get { return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId); }
+            get {
+                if (unique_id == null && type_unique_id == null) {
+                    Log.ErrorFormat ("Creating Source.UniqueId for {0}, but TypeUniqueId is null; trace is {1}", this.Name, System.Environment.StackTrace);
+                }
+                return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId);
+            }
         }
         
-        protected abstract string TypeUniqueId { get; }
+        private string type_unique_id;
+        protected string TypeUniqueId {
+            get { return type_unique_id; }
+            set { type_unique_id = value; }
+        }
 
         public virtual bool CanRename {
             get { return false; }
@@ -549,7 +567,7 @@ namespace Banshee.Sources
             get { return duration_status_formatters.Count; }
         }
         
-        protected virtual int CurrentStatusFormat {
+        public virtual int CurrentStatusFormat {
             get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", 0); }
             set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", value); }
         }
@@ -559,9 +577,14 @@ namespace Banshee.Sources
             return CreateSchema<T> (name, default(T), null, null);
         }
         
-        public SchemaEntry<T> CreateSchema<T> (string name, T defaultValue, string shotDescription, string longDescription)
+        public SchemaEntry<T> CreateSchema<T> (string name, T defaultValue, string shortDescription, string longDescription)
         {
-            return new SchemaEntry<T> (String.Format ("sources.{0}", ConfigurationId), name, defaultValue, shotDescription, longDescription); 
+            return new SchemaEntry<T> (String.Format ("sources.{0}", ConfigurationId), name, defaultValue, shortDescription, longDescription); 
+        }
+        
+        public SchemaEntry<T> CreateSchema<T> (string ns, string name, T defaultValue, string shortDescription, string longDescription)
+        {
+            return new SchemaEntry<T> (String.Format ("sources.{0}.{1}", ConfigurationId, ns), name, defaultValue, shortDescription, longDescription); 
         }
         
         public void CycleStatusFormat ()
