@@ -44,6 +44,7 @@ using Banshee.Collection;
 using Banshee.Sources;
 using Banshee.Collection.Database;
 
+using Hyena;
 using Hyena.Gui;
 using Hyena.Widgets;
 
@@ -90,7 +91,7 @@ namespace Banshee.Gui.Dialogs
             AlbumArtist = track.AlbumArtist ?? String.Empty;
             TrackTitle = track.TrackTitle ?? String.Empty;
             Genre = track.Genre ?? String.Empty;
-            Disc = track.Disc;
+            Disc = track.DiscNumber;
             TrackNumber = track.TrackNumber;
             TrackCount = track.TrackCount;
             Year = track.Year;
@@ -105,7 +106,7 @@ namespace Banshee.Gui.Dialogs
             track.AlbumArtist = AlbumArtist;
             track.TrackTitle = TrackTitle;
             track.Genre = Genre;
-            track.Disc = Disc;
+            track.DiscNumber = Disc;
             track.TrackNumber = TrackNumber;
             track.TrackCount = TrackCount;
             track.Uri = Uri;
@@ -311,8 +312,7 @@ namespace Banshee.Gui.Dialogs
             (Glade["Title"] as Entry).Text = track.TrackTitle;
             (Glade["Genre"] as ComboBoxEntry).Entry.Text = track.Genre;
             
-            (Glade["DurationLabel"] as Label).Text = String.Format ("{0}:{1}", 
-                track.Track.Duration.Minutes, (track.Track.Duration.Seconds).ToString ("00"));
+            (Glade["DurationLabel"] as Label).Text = DateTimeUtil.FormatDuration (track.Track.Duration);
             (Glade["PlayCountLabel"] as Label).Text = track.Track.PlayCount.ToString ();
             (Glade["LastPlayedLabel"] as Label).Text = track.Track.LastPlayed == DateTime.MinValue ?
                 Catalog.GetString ("Never played") : track.Track.LastPlayed.ToString ();
@@ -658,6 +658,10 @@ namespace Banshee.Gui.Dialogs
                 SaveToFile (track);
             }
 
+            if (LibrarySchema.MoveOnInfoSave.Get ()) {
+                MoveSavedFile (track);
+            }
+
             if (track.Track == ServiceManager.PlayerEngine.CurrentTrack) {
                 ServiceManager.PlayerEngine.TrackInfoUpdated ();
             }
@@ -667,6 +671,11 @@ namespace Banshee.Gui.Dialogs
         {
             Banshee.Kernel.Scheduler.Schedule (new SaveTrackMetadataJob (track.Track), Banshee.Kernel.JobPriority.Highest);
         }
+
+        private void MoveSavedFile (EditorTrack track)
+        {
+            Banshee.Kernel.Scheduler.Schedule (new MoveOnInfoSaveJob (track.Track), Banshee.Kernel.JobPriority.Highest);
+        }
     }
 }
 
