@@ -137,7 +137,7 @@ namespace Banshee.Gui
                 current_source = new_source;
             }
             
-            UpdateActions ();
+            Banshee.Base.ThreadAssist.ProxyToMain (UpdateActions);
         }
 
         private void HandleActionsChanged (object sender, EventArgs args)
@@ -152,8 +152,10 @@ namespace Banshee.Gui
 
         private void HandleSelectionChanged (object sender, EventArgs args)
         {
-            OnSelectionChanged ();
-            UpdateActions ();
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                OnSelectionChanged ();
+                UpdateActions ();
+            });
         }
 
         private void HandleEditMenuActivated (object sender, EventArgs args)
@@ -225,7 +227,12 @@ namespace Banshee.Gui
                     
                     UpdateAction ("TrackPropertiesAction", in_database, has_selection, source);
                     UpdateAction ("RateTracksAction", in_database, has_selection, null);
-                    UpdateAction ("AddToPlaylistAction", in_database && primary_source != null && primary_source.SupportsPlaylists, has_selection, null);
+                    UpdateAction ("AddToPlaylistAction", in_database && primary_source != null &&
+                            primary_source.SupportsPlaylists && !primary_source.PlaylistsReadOnly, has_selection, null);
+
+                    if (primary_source != null) {
+                        this["DeleteTracksFromDriveAction"].Label = String.Format (Catalog.GetString ("_Delete From {0}"), primary_source.StorageName);
+                    }
                 }
             } else {
                 Sensitive = Visible = false;
@@ -279,8 +286,9 @@ namespace Banshee.Gui
                 
                 if (handler != null) {
                     handler ();
-                } else {
-                    new TrackEditor (current_source.TrackModel.SelectedItems);
+                } else {  
+                    new Banshee.Gui.Dialogs.TrackEditor (current_source.TrackModel.SelectedItems);
+                    // Banshee.Gui.TrackEditor.TrackEditorDialog.RunEdit (current_source.TrackModel);
                 }
             }
         }
