@@ -37,15 +37,24 @@ using IPod;
 using Hyena;
 using Banshee.Base;
 using Banshee.ServiceStack;
+using Banshee.Sources;
 using Banshee.Dap;
 using Banshee.Hardware;
 using Banshee.Collection.Database;
+using Banshee.Library;
+using Banshee.Playlist;
+
+using Banshee.Dap.Gui;
 
 namespace Banshee.Dap.Ipod
 {
     public class IpodSource : DapSource
     {
         private PodSleuthDevice ipod_device;
+        internal PodSleuthDevice IpodDevice {
+            get { return ipod_device; }
+        }
+        
         private Dictionary<int, IpodTrackInfo> tracks_map = new Dictionary<int, IpodTrackInfo> (); // FIXME: EPIC FAIL
         private bool database_loaded;
         
@@ -59,6 +68,8 @@ namespace Banshee.Dap.Ipod
             get { return database_supported; }
         }
         
+        private UnsupportedDatabaseView unsupported_view;
+        
 #region Device Setup/Dispose
         
         public override void DeviceInitialize (IDevice device)
@@ -69,21 +80,39 @@ namespace Banshee.Dap.Ipod
             if (ipod_device == null) {
                 throw new InvalidDeviceException ();
             }
-            
+
             name_path = Path.Combine (Path.GetDirectoryName (ipod_device.TrackDatabasePath), "BansheeIPodName");
-            
+            Name = GetDeviceName ();
+
             Initialize ();
+
+            AddDapProperty (Catalog.GetString ("Device"), ipod_device.ModelInfo.DeviceClass);
+            AddDapProperty (Catalog.GetString ("Color"), ipod_device.ModelInfo.ShellColor);
+            AddDapProperty (Catalog.GetString ("Generation"), ipod_device.ModelInfo.Generation.ToString ());
+            AddDapProperty (Catalog.GetString ("Capacity"), ipod_device.ModelInfo.AdvertisedCapacity);
+            AddDapProperty (Catalog.GetString ("Serial number"), ipod_device.Serial);
+            AddDapProperty (Catalog.GetString ("Produced on"), ipod_device.ProductionInfo.DisplayDate);
+            AddDapProperty (Catalog.GetString ("Firmware"), ipod_device.FirmwareVersion);
+
+            string [] capabilities = new string [ipod_device.ModelInfo.Capabilities.Count];
+            ipod_device.ModelInfo.Capabilities.CopyTo (capabilities, 0);
+            AddDapProperty (Catalog.GetString ("Capabilities"), String.Join (", ", capabilities));
+            AddYesNoDapProperty (Catalog.GetString ("Supports cover art"), ipod_device.ModelInfo.AlbumArtSupported);
+            AddYesNoDapProperty (Catalog.GetString ("Supports photos"), ipod_device.ModelInfo.PhotosSupported);
         }
-                
+
         public override void Dispose ()
         {
+            ThreadAssist.ProxyToMain (DestroyUnsupportedView);
             CancelSyncThread ();
             base.Dispose ();
         }
 
         // WARNING: This will be called from a thread!
         protected override void Eject ()
-        {   
+        {
+            base.Eject ();
+            CancelSyncThread ();
             if (ipod_device.CanUnmount) {
                 ipod_device.Unmount ();
             }
@@ -108,6 +137,7 @@ namespace Banshee.Dap.Ipod
         {
             LoadIpod ();
             LoadFromDevice (false);
+            OnTracksAdded ();
         }
 
         private void LoadIpod ()
@@ -116,7 +146,7 @@ namespace Banshee.Dap.Ipod
             
             try {
                 if (File.Exists (ipod_device.TrackDatabasePath)) { 
-                    ipod_device.LoadTrackDatabase ();
+                    ipod_device.LoadTrackDatabase (false);
                 } else {
                     int count = CountMusicFiles ();
                     Log.DebugFormat ("Found {0} files in /iPod_Control/Music", count);
@@ -125,14 +155,24 @@ namespace Banshee.Dap.Ipod
                     }
                 }
                 database_supported = true;
+                ThreadAssist.ProxyToMain (DestroyUnsupportedView);
             } catch (DatabaseReadException e) {
                 Log.Exception ("Could not read iPod database", e);
                 ipod_device.LoadTrackDatabase (true);
+                
+                ThreadAssist.ProxyToMain (delegate {
+                    DestroyUnsupportedView ();
+                    unsupported_view = new UnsupportedDatabaseView (this);
+                    unsupported_view.Refresh += OnRebuildDatabaseRefresh;
+                    Properties.Set<Banshee.Sources.Gui.ISourceContents> ("Nereid.SourceContents", unsupported_view);
+                });
             } catch (Exception e) {
                 Log.Exception (e);
             }
             
             database_loaded = true;
+
+            Name = GetDeviceName ();
         }
         
         private int CountMusicFiles ()
@@ -160,16 +200,39 @@ namespace Banshee.Dap.Ipod
             }
             
             tracks_map.Clear ();
-             
+
             if (database_supported || (ipod_device.HasTrackDatabase && 
                 ipod_device.ModelInfo.DeviceClass == "shuffle")) {
                 foreach (Track ipod_track in ipod_device.TrackDatabase.Tracks) {
-                    IpodTrackInfo track = new IpodTrackInfo (ipod_track);
-                    track.PrimarySource = this;
-                    track.Save (false);
-                    tracks_map.Add (track.TrackId, track);
+                    try {
+                        IpodTrackInfo track = new IpodTrackInfo (ipod_track);
+                        track.PrimarySource = this;
+                        track.Save (false);
+                        tracks_map.Add (track.TrackId, track);
+                    } catch (Exception e) {
+                        Log.Exception (e);
+                    }
                 }
-            } 
+
+                Hyena.Data.Sqlite.HyenaSqliteCommand insert_cmd = new Hyena.Data.Sqlite.HyenaSqliteCommand (
+                    @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID)
+                        SELECT ?, TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND ExternalID = ?");
+                foreach (IPod.Playlist playlist in ipod_device.TrackDatabase.Playlists) {
+                    if (playlist.IsOnTheGo) { // || playlist.IsPodcast) {
+                        Console.WriteLine ("have playlist {0} with {1} items but ignoring b/c otg or podcast", playlist.Name, playlist.Tracks.Count);
+                        continue;
+                    }
+                    PlaylistSource pl_src = new PlaylistSource (playlist.Name, this.DbId);
+                    pl_src.Save ();
+                    // We use the IPod.Track.Id here b/c we just shoved it into ExternalID above when we loaded
+                    // the tracks, however when we sync, the Track.Id values may/will change.
+                    foreach (IPod.Track track in playlist.Tracks) {
+                        ServiceManager.DbConnection.Execute (insert_cmd, pl_src.DbId, this.DbId, track.Id);
+                    }
+                    pl_src.UpdateCounts ();
+                    AddChildSource (pl_src);
+                }
+            }
             
             /*else {
                 BuildDatabaseUnsupportedWidget ();
@@ -180,10 +243,30 @@ namespace Banshee.Dap.Ipod
             }*/
         }
         
+        private void OnRebuildDatabaseRefresh (object o, EventArgs args)
+        {
+            ServiceManager.SourceManager.SetActiveSource (MusicGroupSource);
+            base.LoadDeviceContents ();
+        }
+        
+        private void DestroyUnsupportedView ()
+        {
+            if (unsupported_view != null) {
+                unsupported_view.Refresh -= OnRebuildDatabaseRefresh;
+                unsupported_view.Destroy ();
+                unsupported_view = null;
+            }
+        }
+        
 #endregion
 
 #region Source Cosmetics
 
+        internal string [] _GetIconNames ()
+        {
+            return GetIconNames ();
+        }
+
         protected override string [] GetIconNames ()
         {
             string [] names = new string[4];
@@ -251,46 +334,34 @@ namespace Banshee.Dap.Ipod
                 Log.Exception (e);
             }
             
-            this.name = null;
             ipod_device.Name = name;
             base.Rename (name);
         }
         
-        private string name;
-        public override string Name {
-            get {
-                if (name != null) {
-                    return name;
-                }
-                
-                if (File.Exists (name_path)) {
-                    using (StreamReader reader = new StreamReader (name_path, System.Text.Encoding.Unicode)) {
-                        name = reader.ReadLine ();
-                    }
-                }
-                
-                if (String.IsNullOrEmpty (name) && database_loaded && database_supported) {
-                    name = ipod_device.Name;
-                }
-                    
-                if (!String.IsNullOrEmpty (name)) {
-                    return name;
-                } else if (ipod_device.PropertyExists ("volume.label")) {
-                    name = ipod_device.GetPropertyString ("volume.label");
-                } else if (ipod_device.PropertyExists ("info.product")) {
-                    name = ipod_device.GetPropertyString ("info.product");
-                } else {
-                    name = ((IDevice)ipod_device).Name ?? "iPod";
-                }
-                
-                try {
-                    return name;
-                } finally {
-                    if (!database_loaded) {
-                        name = null;
-                    }
+        private string GetDeviceName ()
+        {
+            string name = null;
+            if (File.Exists (name_path)) {
+                using (StreamReader reader = new StreamReader (name_path, System.Text.Encoding.Unicode)) {
+                    name = reader.ReadLine ();
                 }
             }
+            
+            if (String.IsNullOrEmpty (name) && database_loaded && database_supported) {
+                name = ipod_device.Name;
+            }
+                
+            if (!String.IsNullOrEmpty (name)) {
+                return name;
+            } else if (ipod_device.PropertyExists ("volume.label")) {
+                name = ipod_device.GetPropertyString ("volume.label");
+            } else if (ipod_device.PropertyExists ("info.product")) {
+                name = ipod_device.GetPropertyString ("info.product");
+            } else {
+                name = ((IDevice)ipod_device).Name ?? "iPod";
+            }
+            
+            return name;
         }
         
         public override bool CanRename {
@@ -325,9 +396,14 @@ namespace Banshee.Dap.Ipod
         
         public override void Import ()
         {
-            Log.Information ("Import to Library is not implemented for iPods yet", true);
+            Banshee.ServiceStack.ServiceManager.Get<LibraryImportManager> ().Enqueue (Path.Combine (ipod_device.ControlPath, "Music"));
         }
 
+        /*public override void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
+        {
+            throw new Exception ("Copy to Library is not implemented for iPods yet");
+        }*/
+
         protected override void DeleteTrack (DatabaseTrackInfo track)
         {
             lock (sync_mutex) {
@@ -361,6 +437,14 @@ namespace Banshee.Dap.Ipod
             }
         }
 
+        public override void SyncPlaylists ()
+        {
+            if (Monitor.TryEnter (sync_mutex)) {
+                PerformSync ();
+                Monitor.Exit (sync_mutex);
+            }
+        }
+
         private void QueueSync ()
         {
             lock (sync_timeout_mutex) {
@@ -374,12 +458,17 @@ namespace Banshee.Dap.Ipod
         
         private void CancelSyncThread ()
         {
+            Thread thread = sync_thread;
             lock (sync_mutex) {
                 if (sync_thread != null && sync_thread_wait != null) {
                     sync_thread_dispose = true;
                     sync_thread_wait.Set ();
                 }
             }
+
+            if (thread != null) {
+                thread.Join ();
+            }
         }
         
         private bool PerformSync ()
@@ -406,20 +495,24 @@ namespace Banshee.Dap.Ipod
         
         private void PerformSyncThread ()
         {
-            while (true) {
-                sync_thread_wait.WaitOne ();
-                if (sync_thread_dispose) {
-                    break;
+            try {
+                while (true) {
+                    sync_thread_wait.WaitOne ();
+                    if (sync_thread_dispose) {
+                        break;
+                    }
+                    
+                    PerformSyncThreadCycle ();
                 }
                 
-                PerformSyncThreadCycle ();
-            }
-            
-            lock (sync_mutex) {
-                sync_thread_dispose = false;
-                sync_thread_wait.Close ();
-                sync_thread_wait = null;
-                sync_thread = null;
+                lock (sync_mutex) {
+                    sync_thread_dispose = false;
+                    sync_thread_wait.Close ();
+                    sync_thread_wait = null;
+                    sync_thread = null;
+                }
+            } catch (Exception e) {
+                Log.Exception (e);
             }
         }
         
@@ -433,10 +526,13 @@ namespace Banshee.Dap.Ipod
                 
                 try {
                     track.CommitToIpod (ipod_device);
+                    tracks_map[track.TrackId] = track;
                 } catch (Exception e) {
                     Log.Exception ("Cannot save track to iPod", e);
                 }
             }
+
+            // TODO sync updated metadata to changed tracks
             
             while (tracks_to_remove.Count > 0) {
                 IpodTrackInfo track = null;
@@ -455,7 +551,40 @@ namespace Banshee.Dap.Ipod
                 } catch (Exception e) {
                     Log.Exception ("Cannot remove track from iPod", e);
                 }
-            } 
+            }
+
+            // Remove playlists on the device
+            List<IPod.Playlist> device_playlists = new List<IPod.Playlist> (ipod_device.TrackDatabase.Playlists);
+            foreach (IPod.Playlist playlist in device_playlists) {
+                if (!playlist.IsOnTheGo) { // && !playlist.IsPodcast) {
+                    ipod_device.TrackDatabase.RemovePlaylist (playlist);
+                }
+            }
+            device_playlists.Clear ();
+
+            // Add playlists from Banshee to the device
+            foreach (Source child in Children) {
+                PlaylistSource from = child as PlaylistSource;
+                if (from != null && from.Count > 0) {
+                    IPod.Playlist playlist = ipod_device.TrackDatabase.CreatePlaylist (from.Name);
+                    foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
+                        "SELECT CoreTracks.TrackID FROM CoreTracks{0} WHERE {1}",
+                        from.DatabaseTrackModel.JoinFragment, from.DatabaseTrackModel.Condition)))
+                    {
+                        playlist.AddTrack (tracks_map[track_id].IpodTrack);
+                    }
+                }
+            }
+
+            // Sync podcast playlist
+            /*IPod.Playlist podcast_playlist = GetPodcastPlaylist ();
+            podcast_playlist.Clear ();
+            foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (
+                "SELECT CoreTracks.TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND (Attributes & ?) != 0",
+                DbId, (int)Banshee.Collection.TrackMediaAttributes.Podcast))
+            {
+                podcast_playlist.AddTrack (tracks_map[track_id].IpodTrack);
+            }*/
             
             try {
                 ipod_device.TrackDatabase.SaveStarted += OnIpodDatabaseSaveStarted;
@@ -470,6 +599,19 @@ namespace Banshee.Dap.Ipod
                 ipod_device.TrackDatabase.SaveProgressChanged -= OnIpodDatabaseSaveProgressChanged;
             }
         }
+
+        /*private IPod.Playlist GetPodcastPlaylist ()
+        {
+            foreach (IPod.Playlist playlist in ipod_device.TrackDatabase.Playlists) {
+                if (playlist.IsPodcast) {
+                    return playlist;
+                }
+            }
+
+            IPod.Playlist podcast_playlist= ipod_device.TrackDatabase.CreatePlaylist (Catalog.GetString ("Podcasts"));
+            podcast_playlist.IsPodcast = true;
+            return podcast_playlist;
+        }*/
         
         private UserJob sync_user_job;
         
